import argparse
import sys

from zenlog import log


class Parser:

    """Parse the command-line args and return result to the __main__"""

    def __init__(self):
        self.parser = None
        self.result = None

        self.parser = argparse.ArgumentParser(
            description="Play any radio around the globe right from the CLI ",
            prog="radio-active",
            add_help=False,
        )

        self.parser.add_argument(
            "--version", "-V", action="store_true", dest="version", default=False
        )
        self.parser.add_argument(
            "--help",
            "-H",
            action="store_true",
            default=False,
            dest="help",
            help="Show help message",
        )

        self.parser.add_argument(
            "--station",
            "-S",
            action="store",
            dest="station_name",
            help="Specify a station name",
        )

        self.parser.add_argument(
            "--uuid",
            "-U",
            action="store",
            dest="station_uuid",
            help="Specify a station UUID",
        )

        self.parser.add_argument(
            "--log-level",
            "-L",
            action="store",
            default="info",
            dest="log_level",
            help="Specify log level",
        )

        self.parser.add_argument(
            "--discover-by-country",
            "-D",
            action="store",
            dest="discover_country_code",
            help="Discover stations with country code",
        )

        self.parser.add_argument(
            "--discover-by-tag",
            action="store",
            dest="discover_tag",
            help="Discover stations with tag",
        )

        self.parser.add_argument(
            "--discover-by-state",
            action="store",
            dest="discover_state",
            help="Discover stations with state name",
        )

        self.parser.add_argument(
            "--discover-by-language",
            action="store",
            dest="discover_language",
            help="Discover stations with state name",
        )
        self.parser.add_argument(
            "--limit",
            action="store",
            dest="limit",
            default=100,
            help="Limit of entries in discover table",
        )

        self.parser.add_argument(
            "--add-station",
            "-A",
            action="store_true",
            default=False,
            dest="new_station",
            help="Add an entry to your favorite station",
        )

        self.parser.add_argument(
            "--add-to-favorite",
            "-F",
            action="store",
            dest="add_to_favorite",
            help="Save current station to your favorite list",
        )

        self.parser.add_argument(
            "--show-favorite-list",
            "-W",
            action="store_true",
            dest="show_favorite_list",
            default=False,
            help="Show your favorite list in table format",
        )

        self.parser.add_argument(
            "--random",
            "-R",
            action="store_true",
            dest="random",
            default=False,
            help="Play a random station from your favorite list",
        )

        self.parser.add_argument(
            "--flush",
            action="store_true",
            dest="flush",
            default=False,
            help="Flush your favorite list",
        )

        self.parser.add_argument(
            "--volume",
            action="store",
            dest="volume",
            default=80,
            type=int,
            choices=range(0, 101, 10),
            help="Volume to pass down to ffplay",
        )
        self.parser.add_argument(
            "--kill",
            action="store_true",
            dest="kill_ffplays",
            default=False,
            help="kill all the ffplay process initiated by radioactive",
        )

    def parse(self):
        self.result = self.parser.parse_args()

        if self.result is None:
            log.error("Could not parse the arguments properly")
            sys.exit(1)
        return self.result



""" FFplay process handler """

import os
import signal
import subprocess
import sys
import threading
from shutil import which
from time import sleep

import psutil
from zenlog import log


def kill_background_ffplays():
    all_processes = psutil.process_iter(attrs=["pid", "name"])
    count = 0
    # Iterate through the processes and terminate those named "ffplay"
    for process in all_processes:
        try:
            if process.info["name"] == "ffplay":
                pid = process.info["pid"]
                p = psutil.Process(pid)
                p.terminate()
                count += 1
                log.info(f"Terminated ffplay process with PID {pid}")
                if p.is_running():
                    p.kill()
                    log.debug(f"Forcefully killing ffplay process with PID {pid}")
        except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
            # Handle exceptions, such as processes that no longer exist or access denied
            log.debug("Could not terminate a ffplay processes!")
    if count == 0:
        log.info("No background radios are running!")


class Player:

    """FFPlayer handler, it holds all the attributes to properly execute ffplay
    FFmepg required to be installed separately
    """

    def __init__(self, URL, volume):
        self.url = URL
        self.volume = volume
        self.is_playing = False
        self.process = None
        self.exe_path = None
        self.program_name = "ffplay"  # constant value

        log.debug("player: url => {}".format(self.url))
        # check if FFplay is installed
        self.exe_path = which(self.program_name)
        log.debug("FFplay: {}".format(self.exe_path))

        if self.exe_path is None:
            log.critical("FFplay not found, install it first please")
            sys.exit(1)

        self.start_process()

    def start_process(self):
        try:
            self.process = subprocess.Popen(
                [
                    self.exe_path,
                    "-nodisp",
                    "-nostats",
                    "-loglevel",
                    "error",
                    "-volume",
                    f"{self.volume}",
                    self.url,
                ],
                shell=False,
                stdout=subprocess.PIPE,  # Capture standard output
                stderr=subprocess.PIPE,  # Capture standard error
                text=True,  # Use text mode to capture strings
            )
            self.is_running = True
            log.debug("player: ffplay => PID {} initiated".format(self.process.pid))
            # Create a thread to continuously capture and check error output
            error_thread = threading.Thread(target=self.check_error_output)
            error_thread.daemon = True
            error_thread.start()

        except Exception as e:
            # Handle exceptions that might occur during process setup
            log.error("Error while starting radio: {}".format(e))

    def check_error_output(self):
        while self.is_running:
            stderr_result = self.process.stderr.readline()
            if stderr_result:
                log.error("Could not connect to the station")
                try:
                    # try to show the debug info
                    log.debug(stderr_result)
                    # only showing the server response
                    log.error(stderr_result.split(": ")[1])
                except Exception as e:
                    log.debug("Error: {}".format(e))
                    pass

                self.is_running = False
                self.stop()
            sleep(2)

    def terminate_parent_process(self):
        parent_pid = os.getppid()
        print(parent_pid)
        os.kill(parent_pid, signal.SIGINT)

    def is_active(self):
        """Check if the ffplay process is still active."""
        if not self.process:
            log.warning("Process is not initialized")
            return False
        try:
            proc = psutil.Process(self.process.pid)
            if proc.status() == psutil.STATUS_ZOMBIE:
                log.debug("Process is a zombie")
                return False

            if proc.status() == psutil.STATUS_RUNNING:
                return True

            if proc.status() == psutil.STATUS_SLEEPING:
                log.debug("Process is sleeping")
                return True  # Sleeping is considered active for our purpose

            # Handle other process states if needed

            log.warning("Process is not in an expected state")
            return False
        except psutil.NoSuchProcess:
            log.debug("Process not found")
            return False
        except Exception as e:
            log.error("Error while checking process status: {}".format(e))
            return False

    def play(self):
        """Play a station"""
        if not self.is_playing:
            pass  # call the init function again ?

    def stop(self):
        """stop the ffplayer"""

        if self.is_playing:
            try:
                self.process.terminate()  # Terminate the process gracefully
                self.process.wait(timeout=5)  # Wait for process to finish
                log.info("Radio playback stopped successfully")
            except subprocess.TimeoutExpired:
                log.warning("Radio process did not terminate, killing...")
                self.process.kill()  # Kill the process forcefully
            except Exception as e:
                log.error("Error while stopping radio: {}".format(e))
                raise
            finally:
                self.is_playing = False
                self.process = None
        else:
            log.debug("Radio is not currently playing")
            current_pid = os.getpid()
            os.kill(current_pid, signal.SIGINT)



from rich.console import Console
from rich.panel import Panel
from rich.table import Table


def show_help():
    """Show help message as table"""
    console = Console()

    welcome = Panel(
        """
        :radio: Play any radios around the globe right from this Terminal [yellow][blink]:zap:[/blink][/yellow]!
        :smile: Author: Dipankar Pal
        :question: Type '--help' for more details on available commands.
        :bug: Visit https://github.com/deep5050/radio-active to submit issues
        :star: Show some love by starring the project on GitHub [red][blink]:heart:[/blink][/red]
        :dollar: You can donate me at https://deep5050.github.io/payme/
        """,
        title="[b]RADIOACTIVE[/b]",
        width=85,
    )
    console.print(welcome)

    table = Table(show_header=True, header_style="bold magenta")
    table.add_column("Arguments", justify="left")
    table.add_column("Expects value", justify="center")
    table.add_column("Description", justify="left")
    table.add_column("Default", justify="center")
    table.add_column("Note", justify="center")

    table.add_row(
        "--station , -S",
        "yes",
        "A station name to search on the internet",
        "",
        "Optional from second run",
    )
    table.add_row(
        "--uuid , -U",
        "yes",
        "A station UUID to play it directly",
        "",
        "Optional from second run",
    )
    table.add_row(
        "--log-level , -L",
        "yes [info,warning,error,debug]",
        "Log level of the program",
        "info",
        "Optional",
    )
    table.add_row(
        "--add-station , -A",
        "no",
        "Add a  station to your favorite list",
        "False",
        "Optional",
    )
    table.add_row(
        "--add-to-favorite, -F ",
        "yes",
        "Add current station to favorite list with custom name",
        "False",
        "Optional",
    )

    table.add_row(
        "--show-favorite-list, -W ",
        "no",
        "Show your favorite list",
        "False",
        "Optional",
    )

    table.add_row(
        "--discover-by-country, -D",
        "yes",
        "Discover stations by country code",
        "False",
        "Optional",
    )

    table.add_row(
        "--discover-by-state",
        "yes",
        "Discover stations by country state",
        "False",
        "Optional",
    )

    table.add_row(
        "--discover-by-tag",
        "yes",
        "Discover stations by tags/genre",
        "False",
        "Optional",
    )

    table.add_row(
        "--discover-by-language",
        "yes",
        "Discover stations by language",
        "False",
        "Optional",
    )

    table.add_row(
        "--limit",
        "yes",
        "Limit the number of results in discover result table",
        "100",
        "Optional",
    )

    table.add_row(
        "--volume",
        "yes",
        "Volume of the radio between 0 and 100",
        "80",
        "Optional",
    )

    table.add_row("--flush", "no", "Clear your favorite list", "False", "Optional")
    table.add_row("--kill", "no", "Stop background radios", "False", "Optional")

    console.print(table)



#!/usr/bin/env python
import os
import signal
import sys
from time import sleep

from pick import pick
from rich import print
from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from rich.text import Text
from zenlog import log

from radioactive.alias import Alias
from radioactive.app import App
from radioactive.args import Parser
from radioactive.handler import Handler
from radioactive.help import show_help
from radioactive.last_station import Last_station
from radioactive.player import Player, kill_background_ffplays

RED_COLOR = "\033[91m"
END_COLOR = "\033[0m"

# globally needed as signal handler needs it
# to terminate main() properly
player = None


def main():
    log.level("info")
    parser = Parser()
    app = App()
    args = parser.parse()

    # ----------------- all the args ------------- #
    show_help_table = args.help
    station_name = args.station_name
    station_uuid = args.station_uuid
    log_level = args.log_level

    discover_country_code = args.discover_country_code
    discover_state = args.discover_state
    discover_language = args.discover_language
    discover_tag = args.discover_tag

    limit = args.limit
    add_station = args.new_station
    add_to_favorite = args.add_to_favorite
    show_favorite_list = args.show_favorite_list
    flush_fav_list = args.flush
    kill_ffplays = args.kill_ffplays

    VERSION = app.get_version()

    if args.version:
        log.info("RADIO-ACTIVE : version {}".format(VERSION))
        sys.exit(0)

    if show_help_table:
        show_help()
        sys.exit(0)

    if log_level in ["info", "error", "warning", "debug"]:
        log.level(log_level)
    else:
        log.warning("Correct log levels are: error,warning,info(default),debug")

    mode_of_search = ""
    direct_play = False
    direct_play_url = ""
    skip_saving_current_station = False

    # --------------- RICH ------------------- #
    console = Console()

    welcome = Panel(
        """
        :radio: Play any radios around the globe right from this Terminal [yellow][blink]:zap:[/blink][/yellow]!
        :smile: Author: Dipankar Pal
        :question: Type '--help' for more details on available commands
        :bug: Visit: https://github.com/deep5050/radio-active to submit issues
        :star: Show some love by starring the project on GitHub [red][blink]:heart:[/blink][/red]
        :dollar: You can donate me at https://deep5050.github.io/payme/
        :x: Press Ctrl+C to quit
        """,
        title="[b]RADIOACTIVE[/b]",
        width=85,
    )

    print(welcome)

    handler = Handler()
    alias = Alias()
    alias.generate_map()
    last_station = Last_station()

    if app.is_update_available():
        update_msg = (
            "\t[blink]An update available, run [green][italic]pip install radio-active=="
            + app.get_remote_version()
            + "[/italic][/green][/blink]\nSee the changes: https://github.com/deep5050/radio-active/blob/main/CHANGELOG.md"
        )
        update_panel = Panel(
            update_msg,
            width=85,
        )
        print(update_panel)
    else:
        log.debug("Update not available")

    # flush ?
    if flush_fav_list:
        # exit radio after deleting fav stations
        sys.exit(alias.flush())

    # -------------- kill background ffplay PIDs --------------------#
    # sometimes radio exits while ffplay is still running.
    # actively trying to prevent these scenarios. until then use this

    if kill_ffplays:
        kill_background_ffplays()
        sys.exit(0)

    # ----------------- favorite list ---------------- #
    if show_favorite_list:
        log.info("Your favorite station list is below")
        table = Table(show_header=True, header_style="bold magenta")
        table.add_column("Station", justify="left")
        table.add_column("URL / UUID", justify="center")
        if len(alias.alias_map) > 0:
            for entry in alias.alias_map:
                table.add_row(entry["name"], entry["uuid_or_url"])
            console.print(table)
            log.info(f"Your favorite stations are saved in {alias.alias_path}")
        else:
            log.info("You have no favorite station list")
        sys.exit(0)

    # --------------------------- add a station --------------------------#
    if add_station:
        left = input("Enter station name:")
        right = input("Enter station stream-url or radio-browser uuid:")
        if left.strip() == "" or right.strip() == "":
            log.error("Empty inputs not allowed")
            sys.exit(1)
        alias.add_entry(left, right)
        log.info("New entry: {}={} added\n".format(left, right))
        sys.exit(0)

    # ------------------ discover ------------------ #
    _limit_ = int(limit) if limit else 100

    if discover_country_code:
        handler.discover_by_country(discover_country_code, _limit_)

    if discover_state:
        handler.discover_by_state(discover_state, _limit_)

    if discover_language:
        handler.discover_by_language(discover_language, _limit_)

    if discover_tag:
        handler.discover_by_tag(discover_tag, _limit_)

    # -------------------- NOTHING PROVIDED --------------------- #
    # if neither of --station and --uuid provided , look in last_station file

    if station_name is None and station_uuid is None:
        # Add a selection list here. first entry must be the last played station
        # try to fetch the last played station's information

        try:
            last_station_info = last_station.get_info()
        except Exception as e:
            log.debug("Error: {}".format(e))
            # no last station??
            pass

        log.info("You can search for a station on internet using the --station option")
        title = "Please select a station from your favorite list:"
        station_selection_names = []
        station_selection_urls = []

        # add last played station first
        if last_station_info:
            station_selection_names.append(
                f"{last_station_info['name'].strip()} (last played station)"
            )
            try:
                station_selection_urls.append(last_station_info["stationuuid"])
            except Exception as e:
                log.debug("Error: {}".format(e))
                station_selection_urls.append(last_station_info["uuid_or_url"])

        fav_stations = alias.alias_map
        for entry in fav_stations:
            station_selection_names.append(entry["name"].strip())
            station_selection_urls.append(entry["uuid_or_url"])

        options = station_selection_names
        if len(options) == 0:
            # setting message color to red. technically it is not an error though.
            # doing it just to catch user attention :)
            log.info(
                f"{RED_COLOR}No stations to play. please search for a station first!{END_COLOR}"
            )
            sys.exit(0)

        _, index = pick(options, title, indicator="-->")

        # check if there is direct URL or just UUID
        station_option_url = station_selection_urls[index]
        station_name = station_selection_names[index].replace(
            "(last played station)", ""
        )

        if station_option_url.find("://") != -1:
            # set direct play to TRUE
            direct_play = True
            direct_play_url = station_option_url
        else:
            # UUID
            station_uuid = station_option_url

    # --------------------ONLY UUID PROVIDED --------------------- #
    # if --uuid provided call directly
    result = None
    if station_uuid is not None:
        mode_of_search = "uuid"
        log.debug("increased click count for: {}".format(station_uuid))
        handler.vote_for_uuid(station_uuid)

    # ------------------- ONLY STATION PROVIDED ------------------ #

    elif station_name is not None and station_uuid is None:
        # got station name only, looking in alias (if any)

        result = alias.search(station_name)
        if result is not None and alias.found:
            try:
                station_uuid_or_url = result["uuid_or_url"]
                # check if it is a url or a uuid
                if station_uuid_or_url.find("://") != -1:
                    # its a URL
                    log.debug("Entry contains a URL")
                    log.debug("Direct play set to True ")
                    log.info("Current station: {}".format(result["name"]))
                    direct_play = True
                    # assigning url and name directly
                    direct_play_url = result["uuid_or_url"]
                else:
                    log.debug("Entry contains a UUID")
                    # mode_of_search = "uuid"
                    station_uuid = result["uuid_or_url"]  # its a UUID
                    handler.vote_for_uuid(station_uuid)

            except Exception as e:
                log.debug("Error: {}".format(e))
                log.warning("Station found in favorite list but seems to be invalid")
                log.warning("Looking on the web instead")
                alias.found = False

        if alias.found:
            mode_of_search = "uuid"
            if not direct_play:
                log.debug("Looking on the web for given UUID")

        else:
            log.debug("Alias not found, using normal API search")
            mode_of_search = "name"

    if not direct_play:
        # avoid extra API calls since target url is given
        if mode_of_search == "uuid":
            _station_name = handler.play_by_station_uuid(station_uuid)
            station_name = _station_name
        else:
            if not alias.found:
                # when alias was found, we have set the station name to print it correctly,
                # not to do an API call
                handler.play_by_station_name(station_name)

    global player

    target_url = direct_play_url if direct_play else handler.target_station["url"]
    player = Player(target_url, args.volume)

    # writing the station name to a file, next time if user
    # don't specify anything, it will try to start the last station
    last_played_station = {}
    if not alias.found:
        # station was not in the alias file
        last_played_station = handler.target_station
    else:
        last_played_station["name"] = station_name
        last_played_station["uuid_or_url"] = station_uuid_or_url
        last_played_station["alias"] = True

    if not skip_saving_current_station:
        log.debug(f"Saving the current station: {last_played_station}")
        if last_played_station:
            last_station.save_info(last_played_station)

    # TODO: handle error when favouring last played (aliased) station (BUG) (LOW PRIORITY)
    if add_to_favorite:
        try:
            alias.add_entry(add_to_favorite, handler.target_station["url"])
        except Exception as e:
            log.debug("Error: {}".format(e))
            log.error("Could not add to favorite. Already in list?")

    curr_station_name = station_name

    try:
        # TODO fix this. when aliasing a station with an existing name
        # curr_station_name is being None
        panel_station_name = Text(curr_station_name, justify="center")

        station_panel = Panel(
            panel_station_name, title="[blink]:radio:[/blink]", width=85
        )
        console.print(station_panel)
    except Exception as e:
        log.debug("Error: {}".format(e))
        # TODO handle exception
        pass

    if os.name == "nt":
        while True:
            sleep(5)
    else:
        try:
            signal.pause()
        except Exception as e:
            log.debug("Error: {}".format(e))
            pass


def signal_handler(sig, frame):
    global player
    log.debug("You pressed Ctrl+C!")
    log.debug("Stopping the radio")
    if player and player.is_playing:
        player.stop()
    log.info("Exiting now")
    sys.exit(0)


signal.signal(signal.SIGINT, signal_handler)

if __name__ == "__main__":
    main()



"""
    Version of the current program, (in development mode it needs to be updated in every release)
    and to check if an updated version available for the app or not
"""
import json

import requests


class App:
    def __init__(self):
        self.__VERSION__ = "2.6.0"  # change this on every update #
        self.pypi_api = "https://pypi.org/pypi/radio-active/json"
        self.remote_version = ""

    def get_version(self):
        """get the version number as string"""
        return self.__VERSION__

    def get_remote_version(self):
        return self.remote_version

    def is_update_available(self):
        """Checks if the user is using an outdated version of the app,
        if any updates available inform user
        """

        try:
            remote_data = requests.get(self.pypi_api)
            remote_data = remote_data.content.decode("utf8")
            remote_data = json.loads(remote_data)
            self.remote_version = remote_data["info"]["version"]

            # compare two version number
            tup_local = tuple(map(int, self.__VERSION__.split(".")))
            tup_remote = tuple(map(int, self.remote_version.split(".")))

            if tup_remote > tup_local:
                return True
            return False

        except Exception:
            print("Could not fetch remote version number")



""" This module saves the current playing station information to a hidden file,
and loads the data when no arguments are provide """

import json
import os.path

from zenlog import log


class Last_station:

    """Saves the last played radio station information, when user don't provide any -S or -U
    it looks for the information.

    on every successful run, it saves the station information.
    The file it uses to store the data is a hidden file under users' home directory
    """

    def __init__(self):
        self.last_station_path = None

        self.last_station_path = os.path.join(
            os.path.expanduser("~"), ".radio-active-last-station"
        )

    def get_info(self):
        try:
            with open(self.last_station_path, "r") as f:
                last_station = json.load(f)
                return last_station

                # log.info("Playing last station: {}".format(
                #     last_station["name"]))
                # if last_station['alias'] == True:
                #     # if station was an alias
                #     return last_station['uuid_or_url']
                # return last_station["stationuuid"]
        except Exception:
            return ""
            # log.critical("Need a station name  or UUID to play the radio, see help")
            # sys.exit(0)

    def save_info(self, station):
        """dumps the current station information as a json file"""

        log.debug("Dumping station information")
        with open(self.last_station_path, "w") as f:
            json.dump(station, f)



from setuptools import find_packages
from setuptools import setup

from radioactive.app import App
import io

app = App()

DESCRIPTION = "Play any radio around the globe right from the terminal"
VERSION = app.get_version()


def readme():
    with io.open("README.md", "r", encoding="utf-8") as f:
        return f.read()


def required(sfx=""):
    with io.open(f"requirements{sfx}.txt", encoding="utf-8") as f:
        return f.read().splitlines()


setup(
    name="radio-active",
    version=VERSION,
    description=DESCRIPTION,
    long_description=readme(),
    long_description_content_type="text/markdown",
    keywords="pyradios wrapper radios api shortwave internet-radio cli app",
    author="Dipankar Pal",
    author_email="dipankarpal5050@gmail.com",
    url="https://github.com/deep5050/radio-active",
    license="MIT",
    entry_points={
        "console_scripts": [
            "radioactive = radioactive.__main__:main",
            "radio = radioactive.__main__:main",
        ]
    },
    packages=find_packages(exclude=["test*"]),
    install_requires=required(),
    extras_require={"dev": required("-dev")},
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Intended Audience :: End Users/Desktop",
        "Topic :: Multimedia :: Sound/Audio",
    ],
    python_requires=">=3.6",
    project_urls={
        "Source": "https://github.com/deep5050/radio-active/",
        "Upstream": "https://api.radio-browser.info/",
    },
)



import os.path

from zenlog import log


class Alias:
    def __init__(self):
        self.alias_map = []
        self.found = False

        self.alias_path = os.path.join(os.path.expanduser("~"), ".radio-active-alias")

    def generate_map(self):
        """parses the fav list file and generates a list"""
        # create alias map
        if os.path.exists(self.alias_path):
            log.debug(f"Alias file at: {self.alias_path}")
            try:
                with open(self.alias_path, "r+") as f:
                    alias_data = f.read().strip()
                    if alias_data == "":
                        log.debug("Empty alias list")
                        return
                    alias_list = alias_data.splitlines()
                    for alias in alias_list:
                        if alias.strip() == "":
                            # empty line pass
                            continue
                        temp = alias.split("==")
                        left = temp[0]
                        right = temp[1]
                        # may contain both URL and UUID
                        self.alias_map.append({"name": left, "uuid_or_url": right})
            except Exception as e:
                log.debug(f"could not get / parse alias data: {e}")

        else:
            log.debug("Alias file does not exist")

    def search(self, entry):
        """searches for an entry in the fav list with the name
        the right side may contain both url or uuid , need to check properly
        """
        if len(self.alias_map) > 0:
            log.debug("looking under alias file")
            for alias in self.alias_map:
                if alias["name"].strip() == entry.strip():
                    log.debug(
                        "Alias found: {} == {}".format(
                            alias["name"], alias["uuid_or_url"]
                        )
                    )
                    self.found = True
                    return alias

            log.debug("Alias not found")
        else:
            log.debug("Empty Alias file")
        return None

    def add_entry(self, left, right):
        """Adds a new entry to the fav list"""
        if self.search(left) is not None:
            log.warning("An entry with same name already exists, try another name")
            return False
        else:
            with open(self.alias_path, "a+") as f:
                f.write("{}=={}\n".format(left.strip(), right.strip()))
                log.info("Current station added to your favorite list")
            return True

    def flush(self):
        """deletes all the entries in the fav list"""
        try:
            with open(self.alias_path, "w") as f:
                f.flush()
            log.info("All entries deleted in your favorite list")
            return 0
        except Exception as e:
            log.debug("Error: {}".format(e))
            log.error("could not delete your favorite list. something went wrong")
            return 1



"""
    This handler solely depends on pyradios module to communicate with our remote API
"""

import json
import sys

from pyradios import RadioBrowser
from rich.console import Console
from rich.table import Table
from zenlog import log

console = Console()


class Handler:
    """
    radio-browser API handler. This module communicates with the underlying API via PyRadios
    """

    def __init__(self):
        self.API = None
        self.response = None
        self.target_station = None

        # When RadioBrowser can not be initiated properly due to no internet (probably)
        try:
            self.API = RadioBrowser()
        except Exception as e:
            log.debug("Error: {}".format(e))
            log.critical("Something is wrong with your internet connection")
            sys.exit(1)

    def station_validator(self):
        """Validates a response from the API and takes appropriate decision"""

        # when no response from the API
        if not self.response:
            log.error("No stations found by the name")
            sys.exit(0)  # considering it as not an error

        # when multiple results found
        if len(self.response) > 1:
            table = Table(show_header=True, header_style="bold magenta")
            table.add_column("Station", justify="left")
            table.add_column("UUID", justify="center")
            table.add_column("Country", justify="center")
            table.add_column("Tags", justify="center")

            log.warn(
                "{} stations found by the name, select one and run with UUID instead".format(
                    len(self.response)
                )
            )

            for station in self.response:
                table.add_row(
                    station["name"],
                    station["stationuuid"],
                    station["countrycode"],
                    station["tags"],
                )

            console.print(table)
            log.info(
                "If the table does not fit into your screen, \
                \ntry to maximize the window , decrease the font by a bit and retry"
            )

            sys.exit(1)

        # when exactly one response found
        if len(self.response) == 1:
            log.info("Station found: {}".format(self.response[0]["name"].strip()))
            log.debug(json.dumps(self.response[0], indent=3))
            self.target_station = self.response[0]
            # register a valid click to increase its popularity
            self.API.click_counter(self.target_station["stationuuid"])
            # return name
            return self.response[0]["name"].strip()

    # ---------------------------- NAME -------------------------------- #
    def play_by_station_name(self, _name=None):
        """search and play a station by its name"""
        # TODO: handle exact error
        try:
            self.response = self.API.search(name=_name, name_exact=False)
            self.station_validator()
        except Exception as e:
            log.debug("Error: {}".format(e))
            log.error("Something went wrong. please try again.")
            sys.exit(1)

    # ------------------------------- UUID ------------------------------ #
    def play_by_station_uuid(self, _uuid):
        """search and play station by its stationuuid"""
        # TODO: handle exact error
        try:
            self.response = self.API.station_by_uuid(_uuid)
            return self.station_validator()  # should return a station name also
        except Exception as e:
            log.debug("Error: {}".format(e))
            log.error("Something went wrong. please try again.")
            sys.exit(1)

    # ----------------------- ------- COUNTRY -------------------------#
    def discover_by_country(self, _country_code, _limit):
        try:
            discover_result = self.API.search(countrycode=_country_code, limit=_limit)
        except Exception as e:
            log.debug("Error: {}".format(e))
            log.error("Something went wrong. please try again.")
            sys.exit(1)

        if len(discover_result) > 1:
            log.info("Result for country: {}".format(discover_result[0]["country"]))
            table = Table(show_header=True, header_style="bold magenta")
            table.add_column("Station", justify="left")
            table.add_column("UUID", justify="center")
            table.add_column("State", justify="center")
            table.add_column("Tags", justify="center")
            table.add_column("Language", justify="center")

            for res in discover_result:
                table.add_row(
                    res["name"],
                    res["stationuuid"],
                    res["state"],
                    res["tags"],
                    res["language"],
                )
            console.print(table)
            log.info(
                "If the table does not fit into your screen,\
                    \ntry to maximize the window , decrease the font by a bit and retry"
            )

            sys.exit(0)
        else:
            log.error("No stations found for the country code, recheck it")
            sys.exit(1)

    # ------------------- by state ---------------------

    def discover_by_state(self, _state, _limit):
        try:
            discover_result = self.API.search(state=_state, limit=_limit)
        except Exception:
            # print(e)
            log.error("Something went wrong. please try again.")
            sys.exit(1)

        if len(discover_result) > 1:
            table = Table(show_header=True, header_style="bold magenta")
            table.add_column("Station", justify="left")
            table.add_column("UUID", justify="center")
            table.add_column("Country", justify="center")
            table.add_column("Tags", justify="center")
            table.add_column("Language", justify="center")

            for res in discover_result:
                table.add_row(
                    res["name"],
                    res["stationuuid"],
                    res["country"],
                    res["tags"],
                    res["language"],
                )
            console.print(table)
            log.info(
                "If the table does not fit into your screen, \ntry to maximize the window , decrease the font by a bit and retry"
            )

            sys.exit(0)
        else:
            log.error("No stations found for the state, recheck it")
            sys.exit(1)

    # -----------------by language --------------------

    def discover_by_language(self, _language, _limit):
        try:
            discover_result = self.API.search(language=_language, limit=_limit)
        except Exception as e:
            log.debug("Error: {}".format(e))
            log.error("Something went wrong. please try again.")
            sys.exit(1)

        if len(discover_result) > 1:
            table = Table(show_header=True, header_style="bold magenta")
            table.add_column("Station", justify="left")
            table.add_column("UUID", justify="center")
            table.add_column("Country", justify="center")
            table.add_column("Tags", justify="center")

            for res in discover_result:
                table.add_row(
                    res["name"], res["stationuuid"], res["country"], res["tags"]
                )
            console.print(table)
            log.info(
                "If the table does not fit into your screen, \ntry to maximize the window , decrease the font by a bit and retry"
            )

            sys.exit(0)
        else:
            log.error("No stations found for the language, recheck it")
            sys.exit(1)

    # -------------------- by tag ----------------------

    def discover_by_tag(self, _tag, _limit):
        try:
            discover_result = self.API.search(tag=_tag, limit=_limit)
        except Exception as e:
            log.debug("Error: {}".format(e))
            log.error("Something went wrong. please try again.")
            sys.exit(1)

        if len(discover_result) > 1:
            table = Table(show_header=True, header_style="bold magenta")
            table.add_column("Station", justify="left")
            table.add_column("UUID", justify="center")
            table.add_column("country", justify="center")
            table.add_column("Language", justify="center")

            for res in discover_result:
                table.add_row(
                    res["name"], res["stationuuid"], res["country"], res["language"]
                )
            console.print(table)
            log.info(
                "If the table does not fit into your screen, \
                \ntry to maximize the window , decrease the font by a bit and retry"
            )

            sys.exit(0)
        else:
            log.error("No stations found for the tag, recheck it")
            sys.exit(1)

    # ---- increase click count ------------- #
    def vote_for_uuid(self, UUID):
        try:
            result = self.API.click_counter(UUID)
            return result
        except Exception as e:
            log.debug("Something went wrong during increasing click count:{}".format(e))
